//@author: vvvv group
//@help: basic pixel based lightning with point light
//@tags: shading, blinn
//@credits:



struct VertexInputType //VS_IN
{
    float4 Position 	: POSITION;
    float4 UV 			: TEXCOORD0;
	float4 Normal 		: NORMAL;
};

struct PixelInputType // vs2ps
{
    float4 HPosition	: SV_POSITION;
   	float4 UVha			: TEXCOORD0; // dot prods against half-angle
    float4 UVna			: TEXCOORD1; // dot-prods against normal
    float4 UV			: TEXCOORD2; // plain-ol' UV
    float4 diffCol		: COLOR0;
    float4 specCol		: COLOR1;
};

Texture2D halfAngleMap <string uiname="halfAngleMap";>;
Texture2D normalAngleMap <string uiname="normalAngleMap";>;
Texture2D colorTexture <string uiname="colorTexture";>;

SamplerState linearSampler <string uiname="Sampler1 State";>
{
    Filter = MIN_MAG_MIP_LINEAR;
    AddressU = Clamp;
    AddressV = Clamp;
};

cbuffer cbPerDraw : register(b0)
{
	float4x4 tV: VIEW;
	float4x4 tP: PROJECTION;
	float4x4 tVP: VIEWPROJECTION;
}; 

cbuffer cbPerObj : register( b1 )
{
	float4x4 tW : WORLD;
	float4x4 tWIT: WORLDINVERSETRANSPOSE;
	float4x4 tWVP : WORLDVIEWPROJECTION;
	float4x4 tVI: VIEWINVERSE;
	
	float3 LightPos <float uimin=-100.0; float uimax=100.0;> = { 0.00, 5.00, -2.00 };
	
	float Alpha <float uimin=0.0; float uimax=1.0;> = 1; 
	float4 LightColor <bool color=true;String uiname="LightColor";> = { 0.8f,0.8f,1.0f,1.0f };
	float4 SurfColor <bool color=true;String uiname="SurfColor";> = { 1.0f,1.0f,1.0f,1.0f };
	
	float4x4 tColor <string uiname="Color Transform";>;
};

cbuffer cbTextureData : register(b2)
{
	float4x4 tTex <string uiname="Texture Transform"; bool uvspace=true; >;
};

	//Hilight Strength
	float Ks <float uimin=0.0; float uimax=1.0;> = 0.5;

// Vertex Shader Cook-Torrance
////////////////////////////////////////////////////////////////////////////////

PixelInputType CookTorrance_VS(VertexInputType input)
{
    PixelInputType output;

	float3 worldNormal = mul(input.Normal, tWIT).xyz;
    
	worldNormal = normalize(worldNormal);

    //build float4
	float4 Po = float4(input.Position.xyz,1.0);
	
    //compute worldspace position
    float3 worldSpacePos = mul(Po, tW).xyz;
	
    float3 LightVec = normalize(LightPos - worldSpacePos);
	
    float ldn = dot(LightVec,worldNormal);
	
    float diffComp = max(0,ldn);
	
    float4 diffContrib = SurfColor * diffComp * LightColor;

    output.diffCol = diffContrib;
    output.diffCol.w = 1.0;
    output.UV = input.UV;

    float3 EyePos = tVI[3].xyz;
	
    float3 vertToEye = normalize(EyePos - worldSpacePos);
	
    float3 halfAngle = normalize(vertToEye + LightVec);
	
    float4 halfIndices = float4(0.5+dot(LightVec, halfAngle)/2.0,
				1.0 - (0.5+dot(worldNormal, halfAngle)/2.0),0.0,1.0);
	
    float4 normIndices = float4(0.5+dot(LightVec, worldNormal)/2.0,
			   1.0 - (0.5+dot(worldNormal, vertToEye)/2.0),0.0,1.0);
	
    output.UVha = halfIndices;
    output.UVna = normIndices;
	
    output.specCol = float4(0,0,0,1);

    // transform into homogeneous-clip space
    output.HPosition = mul(Po, tWVP);
	
    return output;
}

// Pixel Shader Cook-Torrance
////////////////////////////////////////////////////////////////////////////////

float4 CookTorrance_PS(PixelInputType input) : SV_TARGET

{
    float4 nspec = halfAngleMap.Sample(linearSampler, input.UVha.xy) * 
				   normalAngleMap.Sample(linearSampler, input.UVna.xy) * Ks * LightColor;

    float4 ndiff = colorTexture.Sample(linearSampler, input.UV.xy) * input.diffCol;

	return (ndiff + nspec);
}

////////////////////////////////////////////////////////////////////////////////

technique10 CookTorrance
{
	pass p0
	{
		SetVertexShader( CompileShader( vs_4_0, CookTorrance_VS() ) );
		SetPixelShader( CompileShader( ps_4_0, CookTorrance_PS() ) );
	}
}
