//@author: vvvv group
//@help: basic pixel based lightning with point light
//@tags: shading, blinn
//@credits:



struct VertexInputType //VS_IN
{
    float4 Position 	: POSITION;
    float4 UV 			: TEXCOORD0;
	float4 Normal 		: NORMAL;
};

struct PixelInputType // vs2ps
{
    float4 HPosition	: SV_POSITION;
   	float4 UVha			: TEXCOORD0; // dot prods against half-angle
    float4 UVna			: TEXCOORD1; // dot-prods against normal
    float4 UV			: TEXCOORD2; // plain-ol' UV
    float4 diffCol		: COLOR0;
    float4 specCol		: COLOR1;
};

Texture2D colorTexture <string uiname="colorTexture";>;

SamplerState linearSampler <string uiname="Sampler1 State";>
{
    Filter = MIN_MAG_MIP_LINEAR;
    AddressU = Clamp;
    AddressV = Clamp;
};

TextureCube cubeMap <string uiname="cubeMap";>;

SamplerState cubeMapSampler <string uiname="Sampler2 State";>
{
    Filter = MIN_MAG_MIP_LINEAR;
    AddressU = Clamp;
    AddressV = Clamp;
};

cbuffer cbPerDraw : register(b0)
{
	float4x4 tV: VIEW;
	float4x4 tP: PROJECTION;
	float4x4 tVP: VIEWPROJECTION;
}; 

cbuffer cbPerObj : register( b1 )
{
	float4x4 tW : WORLD;
	float4x4 tWIT: WORLDINVERSETRANSPOSE;
	float4x4 tWVP : WORLDVIEWPROJECTION;
	float4x4 tVI: VIEWINVERSE;
	
	float3 LightPos <float uimin=-100.0; float uimax=100.0;> = { 0.00, 5.00, -2.00 };
	
	float Alpha <float uimin=0.0; float uimax=1.0;> = 1; 

	float4 AmbiColor <bool color=true;String uiname="AmbiColor";> = { 0.1f,0.1f,0.1f,1.0f };
	float4 SurfColor <bool color=true;String uiname="SurfColor";> = { 1.0f,1.0f,1.0f,1.0f };
	
	float4x4 tColor <string uiname="Color Transform";>;
};

cbuffer cbTextureData : register(b2)
{
	float4x4 tTex <string uiname="Texture Transform"; bool uvspace=true; >;
};

	//Fresnel Strength
	float Kr <float uimin=0.0; float uimax=1.0;> = 0.6;
	//Fresnel Expon
	float FresExp <float uimin=1.0; float uimax=10.0;> = 5.0;

// Vertex Shader Fresnel
////////////////////////////////////////////////////////////////////////////////

PixelInputType Fresnel_VS(VertexInputType input)
{
    PixelInputType output;

    float3 worldNormal = mul(input.Normal, tWIT).xyz;
	
    worldNormal = normalize(worldNormal);
	
    float4 Po = float4(input.Position.xyz,1);
	
    //compute worldspace position
    float3 worldSpacePos = mul(Po, tW).xyz;
	
    float3 EyePos = tVI[3].xyz;
	
    float3 vertToEye = normalize(worldSpacePos - EyePos);
	
    float fake = abs(dot(vertToEye,worldNormal));
	
    fake = Kr * (1.0 - pow(fake,FresExp));
	
    output.UVha.xyz = reflect(vertToEye,worldNormal);
    output.UVha.w = 1.0;
    output.UVna = input.UV;
    output.UV = input.UV;
    output.diffCol = SurfColor * AmbiColor;
    output.specCol = float4(fake.xxx,1.0);
    output.HPosition = mul(Po, tWVP);
	
    return output;
}

// Pixel Shader Fresnel
////////////////////////////////////////////////////////////////////////////////

float4 Fresnel_PS(PixelInputType input) : SV_TARGET

{
    float4 env = cubeMap.Sample(cubeMapSampler, input.UVha.xyz);
    return (input.diffCol * colorTexture.Sample(linearSampler,input.UVna.xy) + input.specCol * env);
}

technique10 Fresnel 
{
    pass p0
	{
		SetVertexShader( CompileShader( vs_4_0, Fresnel_VS() ) );
		SetPixelShader( CompileShader( ps_4_0, Fresnel_PS() ) );
	}
}
